﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight;
using System.IO;
using System.Windows;
using System.Collections.ObjectModel;
using GalaSoft.MvvmLight.Command;
using System.Diagnostics;

namespace CloseAbout.ViewModels
{
	public class AboutViewModel: ViewModelBase
	{
		private ObservableCollection<string> _versionList;
		private Visibility _isVisible = Visibility.Collapsed;

		#region RelayCommands
		public RelayCommand RefreshListCommand
		{
			get;
			set;
		}

		public RelayCommand CopyCommand
		{
			get;
			set;
		}
		#endregion RelayCommands

		#region Properties
		string sFile
		{
			get; set;
		}
		public ObservableCollection<string> VersionList
		{
			get => _versionList;
			set
			{
				if (_versionList == value)
				{
					return;
				}
				_versionList = value;
				RaisePropertyChanged(nameof(VersionList));//This name must match the name in the "ItemsSource="{Binding MrsUserDataTable}" of MrsUserSettings.xaml
														  //for this to work.
			}
		}

		public Visibility IsVisible
		{
			get => _isVisible;
			set
			{
				_isVisible = value;
				RaisePropertyChanged(nameof(IsVisible));
			}
		}
		#endregion Properties
		public AboutViewModel()
		{
			RefreshListCommand = new RelayCommand(OnRefreshListCommandExecute, () => true);
			CopyCommand = new RelayCommand(OnCopyCommand, () => true);
		}

		private List<string> GetProgramData()
		{
			DirectoryInfo fSet = new DirectoryInfo(System.AppDomain.CurrentDomain.BaseDirectory);
			string ProgramPath = Path.GetDirectoryName(fSet.ToString());

			//get list of dlls and exe for display
			FileInfo[] saFiles = fSet.GetFiles();
			string[] aFiles = new string[saFiles.Count()];
			int i = 0;

			foreach (FileInfo f in saFiles)
			{
				aFiles[i] = ProgramPath + "\\" + f.Name;
				i++;
			}
			return GetVersion(aFiles);
		}


		/// <summary>GetVersion(string[] aFiles)
		/// Receive the array list of files found in the application path to reach "dll" and "exe"
		/// version details to put into the user control's RadListbox.
		/// Using Telerik's list box to show the file list.
		/// </summary>
		/// <param name="aFiles">
		/// An array of the files found in the program path
		/// </param>
		/// <returns>
		/// List<string> lsVersionList to the RadListbox control
		/// </returns>
		private List<string> GetVersion(string[] aFiles)
		{
			List<string> lsVersionList = new List<string>();
			FileVersionInfo vmaFileVersion = null;

			foreach (string sService in aFiles)
			{
				if (sService.IndexOf(".exe") > 0)
				{
					vmaFileVersion = FileVersionInfo.GetVersionInfo(sService);
					lsVersionList.Add(Path.GetFileName(vmaFileVersion.FileName) + "\t" + vmaFileVersion.FileVersion);
				}
				if (sService.IndexOf(".dll") > 0)
				{
					vmaFileVersion = FileVersionInfo.GetVersionInfo(sService);
					lsVersionList.Add(Path.GetFileName(vmaFileVersion.FileName) + "\t" + vmaFileVersion.FileVersion);
				}
			}

			return lsVersionList;
		}

		public void OnRefreshListCommandExecute()
		{
			VersionList = new ObservableCollection<string>(GetProgramData());
		}		

		public void CleanUp()
		{
			if (_versionList != null)
				_versionList.Clear();
		}

		public void OnCopyCommand()
		{
			//debug
			using (StreamWriter output = new StreamWriter("Version Details.txt"))
			{
				output.WriteLine("VoicemailAnalytics file details:");
				foreach (string sList in _versionList)
				{
					output.WriteLine(sList, output);
				}
				output.Close();
			}
			//
		}



		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}
}
